@@ -4,22 +4,28 @@
 
 # Interpreting linear constraints like "2*x1 + x2 = 0"
 
-from __future__ import print_function
-
 # These are made available in the patsy.* namespace
 __all__ = ["LinearConstraint"]
 
 import re
-import six
+
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping
 import numpy as np
 from patsy import PatsyError
 from patsy.origin import Origin
-from patsy.util import (atleast_2d_column_default,
-                        repr_pretty_delegate, repr_pretty_impl,
-                        SortAnythingKey,
-                        no_pickling, assert_no_pickling)
-from patsy.infix_parser import Token, Operator, ParseNode, infix_parse
-from patsy.compat import Scanner, Mapping
+from patsy.util import (
+    atleast_2d_column_default,
+    repr_pretty_delegate,
+    repr_pretty_impl,
+    no_pickling,
+    assert_no_pickling,
+)
+from patsy.infix_parser import Token, Operator, infix_parse
+from patsy.parse_formula import _parsing_error_test
+
 
 class LinearConstraint(object):
     """A linear constraint in matrix form.
@@ -43,6 +49,7 @@ class LinearConstraint(object):
        A list of strings giving the names of the variables being
        constrained. (Used only for consistency checking.)
     """
+
     def __init__(self, variable_names, coefs, constants=None):
         self.variable_names = list(variable_names)
         self.coefs = np.atleast_2d(np.asarray(coefs, dtype=float))
@@ -58,14 +65,14 @@ class LinearConstraint(object):
             raise ValueError("must have at least one row in constraint matrix")
         if self.coefs.shape[0] != self.constants.shape[0]:
             raise ValueError("shape mismatch between coefs and constants")
-        if np.any(np.all(self.coefs == 0, axis=1)):
-            raise ValueError("can't test a constant constraint")
 
     __repr__ = repr_pretty_delegate
+
     def _repr_pretty_(self, p, cycle):
         assert not cycle
-        return repr_pretty_impl(p, self,
-                                [self.variable_names, self.coefs, self.constants])
+        return repr_pretty_impl(
+            p, self, [self.variable_names, self.coefs, self.constants]
+        )
 
     __getstate__ = no_pickling
 
@@ -84,12 +91,16 @@ class LinearConstraint(object):
         for constraint in constraints:
             if constraint.variable_names != variable_names:
                 raise ValueError("variable names don't match")
-        coefs = np.row_stack([c.coefs for c in constraints])
-        constants = np.row_stack([c.constants for c in constraints])
+        coefs = np.vstack([c.coefs for c in constraints])
+        constants = np.vstack([c.constants for c in constraints])
         return cls(variable_names, coefs, constants)
 
+
 def test_LinearConstraint():
-    from numpy.testing.utils import assert_equal
+    try:
+        from numpy.testing import assert_equal
+    except ImportError:
+        from numpy.testing.utils import assert_equal
     lc = LinearConstraint(["foo", "bar"], [1, 1])
     assert lc.variable_names == ["foo", "bar"]
     assert_equal(lc.coefs, [[1, 1]])
@@ -98,64 +109,80 @@ def test_LinearConstraint():
     lc = LinearConstraint(["foo", "bar"], [[1, 1], [2, 3]], [10, 20])
     assert_equal(lc.coefs, [[1, 1], [2, 3]])
     assert_equal(lc.constants, [[10], [20]])
-    
+
     assert lc.coefs.dtype == np.dtype(float)
     assert lc.constants.dtype == np.dtype(float)
 
-    from nose.tools import assert_raises
-    assert_raises(ValueError, LinearConstraint, ["a"], [[1, 2]])
-    assert_raises(ValueError, LinearConstraint, ["a"], [[[1]]])
-    assert_raises(ValueError, LinearConstraint, ["a"], [[1, 2]], [3, 4])
-    assert_raises(ValueError, LinearConstraint, ["a", "b"], [[1, 2]], [3, 4])
-    assert_raises(ValueError, LinearConstraint, ["a"], [[0]])
-    assert_raises(ValueError, LinearConstraint, ["a"], [[1]], [[]])
-    assert_raises(ValueError, LinearConstraint, ["a", "b"], [])
-    assert_raises(ValueError, LinearConstraint, ["a", "b"],
-                  np.zeros((0, 2)))
+    # statsmodels wants to be able to create degenerate constraints like this,
+    # see:
+    #     https://github.com/pydata/patsy/issues/89
+    # We used to forbid it, but I guess it's harmless, so why not.
+    lc = LinearConstraint(["a"], [[0]])
+    assert_equal(lc.coefs, [[0]])
+
+    import pytest
+
+    pytest.raises(ValueError, LinearConstraint, ["a"], [[1, 2]])
+    pytest.raises(ValueError, LinearConstraint, ["a"], [[[1]]])
+    pytest.raises(ValueError, LinearConstraint, ["a"], [[1, 2]], [3, 4])
+    pytest.raises(ValueError, LinearConstraint, ["a", "b"], [[1, 2]], [3, 4])
+    pytest.raises(ValueError, LinearConstraint, ["a"], [[1]], [[]])
+    pytest.raises(ValueError, LinearConstraint, ["a", "b"], [])
+    pytest.raises(ValueError, LinearConstraint, ["a", "b"], np.zeros((0, 2)))
 
     assert_no_pickling(lc)
 
+
 def test_LinearConstraint_combine():
-    comb = LinearConstraint.combine([LinearConstraint(["a", "b"], [1, 0]),
-                                     LinearConstraint(["a", "b"], [0, 1], [1])])
+    comb = LinearConstraint.combine(
+        [
+            LinearConstraint(["a", "b"], [1, 0]),
+            LinearConstraint(["a", "b"], [0, 1], [1]),
+        ]
+    )
     assert comb.variable_names == ["a", "b"]
-    from numpy.testing.utils import assert_equal
+    try:
+        from numpy.testing import assert_equal
+    except ImportError:
+        from numpy.testing.utils import assert_equal
     assert_equal(comb.coefs, [[1, 0], [0, 1]])
     assert_equal(comb.constants, [[0], [1]])
 
-    from nose.tools import assert_raises
-    assert_raises(ValueError, LinearConstraint.combine, [])
-    assert_raises(ValueError, LinearConstraint.combine,
-                  [LinearConstraint(["a"], [1]), LinearConstraint(["b"], [1])])
-    
+    import pytest
+
+    pytest.raises(ValueError, LinearConstraint.combine, [])
+    pytest.raises(
+        ValueError,
+        LinearConstraint.combine,
+        [LinearConstraint(["a"], [1]), LinearConstraint(["b"], [1])],
+    )
+
 
 _ops = [
     Operator(",", 2, -100),
-
     Operator("=", 2, 0),
-    
     Operator("+", 1, 100),
     Operator("-", 1, 100),
     Operator("+", 2, 100),
     Operator("-", 2, 100),
-
     Operator("*", 2, 200),
     Operator("/", 2, 200),
-    ]
+]
 
 _atomic = ["NUMBER", "VARIABLE"]
 
+
 def _token_maker(type, string):
     def make_token(scanner, token_string):
         if type == "__OP__":
             actual_type = token_string
         else:
             actual_type = type
-        return Token(actual_type,
-                     Origin(string, *scanner.match.span()),
-                     token_string)
+        return Token(actual_type, Origin(string, *scanner.match.span()), token_string)
+
     return make_token
 
+
 def _tokenize_constraint(string, variable_names):
     lparen_re = r"\("
     rparen_re = r"\)"
@@ -174,37 +201,42 @@ def _tokenize_constraint(string, variable_names):
         (variable_re, _token_maker("VARIABLE", string)),
         (num_re, _token_maker("NUMBER", string)),
         (whitespace_re, None),
-        ]
+    ]
 
-    scanner = Scanner(lexicon)
+    scanner = re.Scanner(lexicon)
     tokens, leftover = scanner.scan(string)
     if leftover:
         offset = len(string) - len(leftover)
-        raise PatsyError("unrecognized token in constraint",
-                            Origin(string, offset, offset + 1))
+        raise PatsyError(
+            "unrecognized token in constraint", Origin(string, offset, offset + 1)
+        )
 
     return tokens
 
+
 def test__tokenize_constraint():
     code = "2 * (a + b) = q"
     tokens = _tokenize_constraint(code, ["a", "b", "q"])
-    expecteds = [("NUMBER", 0, 1, "2"),
-                 ("*", 2, 3, "*"),
-                 (Token.LPAREN, 4, 5, "("),
-                 ("VARIABLE", 5, 6, "a"),
-                 ("+", 7, 8, "+"),
-                 ("VARIABLE", 9, 10, "b"),
-                 (Token.RPAREN, 10, 11, ")"),
-                 ("=", 12, 13, "="),
-                 ("VARIABLE", 14, 15, "q")]
+    expecteds = [
+        ("NUMBER", 0, 1, "2"),
+        ("*", 2, 3, "*"),
+        (Token.LPAREN, 4, 5, "("),
+        ("VARIABLE", 5, 6, "a"),
+        ("+", 7, 8, "+"),
+        ("VARIABLE", 9, 10, "b"),
+        (Token.RPAREN, 10, 11, ")"),
+        ("=", 12, 13, "="),
+        ("VARIABLE", 14, 15, "q"),
+    ]
     for got, expected in zip(tokens, expecteds):
         assert isinstance(got, Token)
         assert got.type == expected[0]
         assert got.origin == Origin(code, expected[1], expected[2])
         assert got.extra == expected[3]
 
-    from nose.tools import assert_raises
-    assert_raises(PatsyError, _tokenize_constraint, "1 + @b", ["b"])
+    import pytest
+
+    pytest.raises(PatsyError, _tokenize_constraint, "1 + @b", ["b"])
     # Shouldn't raise an error:
     _tokenize_constraint("1 + @b", ["@b"])
 
@@ -221,9 +253,10 @@ def test__tokenize_constraint():
     assert [t.type for t in tokens] == ["NUMBER", "*", "VARIABLE", ","]
     assert [t.extra for t in tokens] == ["2", "*", "a[1,1]", ","]
 
+
 def parse_constraint(string, variable_names):
-    return infix_parse(_tokenize_constraint(string, variable_names),
-                       _ops, _atomic)
+    return infix_parse(_tokenize_constraint(string, variable_names), _ops, _atomic)
+
 
 class _EvalConstraint(object):
     def __init__(self, variable_names):
@@ -241,7 +274,7 @@ class _EvalConstraint(object):
             ("/", 2): self._eval_binary_div,
             ("=", 2): self._eval_binary_eq,
             (",", 2): self._eval_binary_comma,
-            }
+        }
 
     # General scheme: there are 2 types we deal with:
     #   - linear combinations ("lincomb"s) of variables and constants,
@@ -251,7 +284,7 @@ class _EvalConstraint(object):
     #   - LinearConstraint objects
 
     def is_constant(self, coefs):
-        return np.all(coefs[:self._N] == 0)
+        return np.all(coefs[: self._N] == 0)
 
     def _eval_variable(self, tree):
         var = tree.token.extra
@@ -280,8 +313,9 @@ class _EvalConstraint(object):
         left = self.eval(tree.args[0])
         right = self.eval(tree.args[1])
         if not self.is_constant(right):
-            raise PatsyError("Can't divide by a variable in a linear "
-                                "constraint", tree.args[1])
+            raise PatsyError(
+                "Can't divide by a variable in a linear " "constraint", tree.args[1]
+            )
         return left / right[-1]
 
     def _eval_binary_multiply(self, tree):
@@ -292,8 +326,9 @@ class _EvalConstraint(object):
         elif self.is_constant(right):
             return left * right[-1]
         else:
-            raise PatsyError("Can't multiply one variable by another "
-                                "in a linear constraint", tree)
+            raise PatsyError(
+                "Can't multiply one variable by another " "in a linear constraint", tree
+            )
 
     def _eval_binary_eq(self, tree):
         # Handle "a1 = a2 = a3", which is parsed as "(a1 = a2) = a3"
@@ -307,7 +342,7 @@ class _EvalConstraint(object):
                 args[i] = arg.args[1 - i]
         left = self.eval(args[0])
         right = self.eval(args[1])
-        coefs = left[:self._N] - right[:self._N]
+        coefs = left[: self._N] - right[: self._N]
         if np.all(coefs == 0):
             raise PatsyError("no variables appear in constraint", tree)
         constant = -left[-1] + right[-1]
@@ -330,48 +365,44 @@ class _EvalConstraint(object):
                 return val
             else:
                 assert val.size == self._N + 1
-                if np.all(val[:self._N] == 0):
-                    raise PatsyError("term is constant, with no variables",
-                                        tree)
-                return LinearConstraint(self._variable_names,
-                                        val[:self._N],
-                                        -val[-1])
+                if np.all(val[: self._N] == 0):
+                    raise PatsyError("term is constant, with no variables", tree)
+                return LinearConstraint(self._variable_names, val[: self._N], -val[-1])
         else:
             # Force it to *not* be a constraint
             if isinstance(val, LinearConstraint):
                 raise PatsyError("unexpected constraint object", tree)
             return val
 
+
 def linear_constraint(constraint_like, variable_names):
     """This is the internal interface implementing
     DesignInfo.linear_constraint, see there for docs."""
     if isinstance(constraint_like, LinearConstraint):
         if constraint_like.variable_names != variable_names:
-            raise ValueError("LinearConstraint has wrong variable_names "
-                             "(got %r, expected %r)"
-                             % (constraint_like.variable_names,
-                                variable_names))
+            raise ValueError(
+                "LinearConstraint has wrong variable_names "
+                "(got %r, expected %r)"
+                % (constraint_like.variable_names, variable_names)
+            )
         return constraint_like
 
     if isinstance(constraint_like, Mapping):
         # Simple conjunction-of-equality constraints can be specified as
         # dicts. {"x": 1, "y": 2} -> tests x = 1 and y = 2. Keys can be
         # either variable names, or variable indices.
-        coefs = np.zeros((len(constraint_like), len(variable_names)),
-                         dtype=float)
+        coefs = np.zeros((len(constraint_like), len(variable_names)), dtype=float)
         constants = np.zeros(len(constraint_like))
         used = set()
-        for i, (name, value) in enumerate(six.iteritems(constraint_like)):
+        for i, (name, value) in enumerate(constraint_like.items()):
             if name in variable_names:
                 idx = variable_names.index(name)
-            elif isinstance(name, six.integer_types):
+            elif isinstance(name, int):
                 idx = name
             else:
-                raise ValueError("unrecognized variable name/index %r"
-                                 % (name,))
+                raise ValueError("unrecognized variable name/index %r" % (name,))
             if idx in used:
-                raise ValueError("duplicated constraint on %r"
-                                 % (variable_names[idx],))
+                raise ValueError("duplicated constraint on %r" % (variable_names[idx],))
             used.add(idx)
             coefs[i, idx] = 1
             constants[i] = value
@@ -381,9 +412,11 @@ def linear_constraint(constraint_like, variable_names):
         constraint_like = [constraint_like]
         # fall-through
 
-    if (isinstance(constraint_like, list)
+    if (
+        isinstance(constraint_like, list)
         and constraint_like
-        and isinstance(constraint_like[0], str)):
+        and isinstance(constraint_like[0], str)
+    ):
         constraints = []
         for code in constraint_like:
             if not isinstance(code, str):
@@ -403,8 +436,12 @@ def linear_constraint(constraint_like, variable_names):
     coefs = np.asarray(constraint_like, dtype=float)
     return LinearConstraint(variable_names, coefs)
 
+
 def _check_lincon(input, varnames, coefs, constants):
-    from numpy.testing.utils import assert_equal
+    try:
+        from numpy.testing import assert_equal
+    except ImportError:
+        from numpy.testing.utils import assert_equal
     got = linear_constraint(input, varnames)
     print("got", got)
     expected = LinearConstraint(varnames, coefs, constants)
@@ -415,30 +452,29 @@ def _check_lincon(input, varnames, coefs, constants):
     assert_equal(got.coefs.dtype, np.dtype(float))
     assert_equal(got.constants.dtype, np.dtype(float))
 
+
 def test_linear_constraint():
-    from nose.tools import assert_raises
+    import pytest
     from patsy.compat import OrderedDict
+
     t = _check_lincon
 
     t(LinearConstraint(["a", "b"], [2, 3]), ["a", "b"], [[2, 3]], [[0]])
-    assert_raises(ValueError, linear_constraint,
-                  LinearConstraint(["b", "a"], [2, 3]),
-                  ["a", "b"])
+    pytest.raises(
+        ValueError, linear_constraint, LinearConstraint(["b", "a"], [2, 3]), ["a", "b"]
+    )
 
     t({"a": 2}, ["a", "b"], [[1, 0]], [[2]])
-    t(OrderedDict([("a", 2), ("b", 3)]),
-      ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
-    t(OrderedDict([("a", 2), ("b", 3)]),
-      ["b", "a"], [[0, 1], [1, 0]], [[2], [3]])
+    t(OrderedDict([("a", 2), ("b", 3)]), ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
+    t(OrderedDict([("a", 2), ("b", 3)]), ["b", "a"], [[0, 1], [1, 0]], [[2], [3]])
 
     t({0: 2}, ["a", "b"], [[1, 0]], [[2]])
     t(OrderedDict([(0, 2), (1, 3)]), ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
 
-    t(OrderedDict([("a", 2), (1, 3)]),
-      ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
+    t(OrderedDict([("a", 2), (1, 3)]), ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
 
-    assert_raises(ValueError, linear_constraint, {"q": 1}, ["a", "b"])
-    assert_raises(ValueError, linear_constraint, {"a": 1, 0: 2}, ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, {"q": 1}, ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, {"a": 1, 0: 2}, ["a", "b"])
 
     t(np.array([2, 3]), ["a", "b"], [[2, 3]], [[0]])
     t(np.array([[2, 3], [4, 5]]), ["a", "b"], [[2, 3], [4, 5]], [[0], [0]])
@@ -452,43 +488,53 @@ def test_linear_constraint():
 
     t(["a = 2", "b = 3"], ["a", "b"], [[1, 0], [0, 1]], [[2], [3]])
 
-    assert_raises(ValueError, linear_constraint, ["a", {"b": 0}], ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, ["a", {"b": 0}], ["a", "b"])
 
     # Actual evaluator tests
-    t("2 * (a + b/3) + b + 2*3/4 = 1 + 2*3", ["a", "b"],
-      [[2, 2.0/3 + 1]], [[7 - 6.0/4]])
+    t(
+        "2 * (a + b/3) + b + 2*3/4 = 1 + 2*3",
+        ["a", "b"],
+        [[2, 2.0 / 3 + 1]],
+        [[7 - 6.0 / 4]],
+    )
     t("+2 * -a", ["a", "b"], [[-2, 0]], [[0]])
     t("a - b, a + b = 2", ["a", "b"], [[1, -1], [1, 1]], [[0], [2]])
-    t("a = 1, a = 2, a = 3", ["a", "b"],
-      [[1, 0], [1, 0], [1, 0]], [[1], [2], [3]])
+    t("a = 1, a = 2, a = 3", ["a", "b"], [[1, 0], [1, 0], [1, 0]], [[1], [2], [3]])
     t("a * 2", ["a", "b"], [[2, 0]], [[0]])
     t("-a = 1", ["a", "b"], [[-1, 0]], [[1]])
     t("(2 + a - a) * b", ["a", "b"], [[0, 2]], [[0]])
 
     t("a = 1 = b", ["a", "b"], [[1, 0], [0, -1]], [[1], [-1]])
     t("a = (1 = b)", ["a", "b"], [[0, -1], [1, 0]], [[-1], [1]])
-    t("a = 1, a = b = c", ["a", "b", "c"],
-      [[1, 0, 0], [1, -1, 0], [0, 1, -1]], [[1], [0], [0]])
+    t(
+        "a = 1, a = b = c",
+        ["a", "b", "c"],
+        [[1, 0, 0], [1, -1, 0], [0, 1, -1]],
+        [[1], [0], [0]],
+    )
 
     # One should never do this of course, but test that it works anyway...
     t("a + 1 = 2", ["a", "a + 1"], [[0, 1]], [[2]])
 
     t(([10, 20], [30]), ["a", "b"], [[10, 20]], [[30]])
-    t(([[10, 20], [20, 40]], [[30], [35]]), ["a", "b"],
-      [[10, 20], [20, 40]], [[30], [35]])
+    t(
+        ([[10, 20], [20, 40]], [[30], [35]]),
+        ["a", "b"],
+        [[10, 20], [20, 40]],
+        [[30], [35]],
+    )
     # wrong-length tuple
-    assert_raises(ValueError, linear_constraint,
-                  ([1, 0], [0], [0]), ["a", "b"])
-    assert_raises(ValueError, linear_constraint, ([1, 0],), ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, ([1, 0], [0], [0]), ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, ([1, 0],), ["a", "b"])
 
     t([10, 20], ["a", "b"], [[10, 20]], [[0]])
     t([[10, 20], [20, 40]], ["a", "b"], [[10, 20], [20, 40]], [[0], [0]])
     t(np.array([10, 20]), ["a", "b"], [[10, 20]], [[0]])
-    t(np.array([[10, 20], [20, 40]]), ["a", "b"],
-      [[10, 20], [20, 40]], [[0], [0]])
+    t(np.array([[10, 20], [20, 40]]), ["a", "b"], [[10, 20], [20, 40]], [[0], [0]])
 
     # unknown object type
-    assert_raises(ValueError, linear_constraint, None, ["a", "b"])
+    pytest.raises(ValueError, linear_constraint, None, ["a", "b"])
+
 
 _parse_eval_error_tests = [
     # Bad token
@@ -507,8 +553,9 @@ _parse_eval_error_tests = [
     "a = 1, <(a, b)> + 2, c",
 ]
 
-from patsy.parse_formula import _parsing_error_test
+
 def test_eval_errors():
     def doit(bad_code):
         return linear_constraint(bad_code, ["a", "b", "c"])
+
     _parsing_error_test(doit, _parse_eval_error_tests)
