@@ -26,29 +26,41 @@
 
 from functools import wraps
 import numpy as np
-from patsy.util import (atleast_2d_column_default,
-                        asarray_or_pandas, pandas_friendly_reshape,
-                        wide_dtype_for, safe_issubdtype,
-                        no_pickling, assert_no_pickling)
+from patsy.util import (
+    atleast_2d_column_default,
+    asarray_or_pandas,
+    pandas_friendly_reshape,
+    wide_dtype_for,
+    safe_issubdtype,
+    no_pickling,
+    assert_no_pickling,
+)
 
 # These are made available in the patsy.* namespace
-__all__ = ["stateful_transform",
-           "center", "standardize", "scale",
-           ]
+__all__ = [
+    "stateful_transform",
+    "center",
+    "standardize",
+    "scale",
+]
+
 
 def stateful_transform(class_):
     """Create a stateful transform callable object from a class that fulfills
     the :ref:`stateful transform protocol <stateful-transform-protocol>`.
     """
+
     @wraps(class_)
     def stateful_transform_wrapper(*args, **kwargs):
         transform = class_()
         transform.memorize_chunk(*args, **kwargs)
         transform.memorize_finish()
         return transform.transform(*args, **kwargs)
+
     stateful_transform_wrapper.__patsy_stateful_transform__ = class_
     return stateful_transform_wrapper
 
+
 # class NonIncrementalStatefulTransform(object):
 #     def __init__(self):
 #         self._data = []
@@ -76,6 +88,7 @@ def stateful_transform(class_):
 # class QuantileEstimatingTransform(NonIncrementalStatefulTransform):
 #     def memorize_all(self, input_data, *args, **kwargs):
 
+
 class Center(object):
     """center(x)
 
@@ -85,6 +98,7 @@ class Center(object):
 
     Equivalent to ``standardize(x, rescale=False)``
     """
+
     def __init__(self):
         self._sum = None
         self._count = 0
@@ -118,8 +132,10 @@ class Center(object):
 
     __getstate__ = no_pickling
 
+
 center = stateful_transform(Center)
 
+
 # See:
 #   http://en.wikipedia.org/wiki/Algorithms_for_calculating_variance#On-line_algorithm
 # or page 232 of Knuth vol. 3 (3rd ed.).
@@ -141,6 +157,7 @@ class Standardize(object):
        memory-efficient online algorithm, making it suitable for use with
        large incrementally processed data-sets.
     """
+
     def __init__(self):
         self.current_n = 0
         self.current_mean = None
@@ -176,6 +193,7 @@ class Standardize(object):
 
     __getstate__ = no_pickling
 
+
 standardize = stateful_transform(Standardize)
 # R compatibility:
 scale = standardize
